@@ -334,6 +334,9 @@ class atxServer(MongoClientTradeGateway):
         mudan_id = str(record.QuoteId).strip(' ')
         if not oid in self.oid_to_acc: #shut_down or other problem
             trade_acc = str(record.ClientName).strip(' ')
+            if trade_acc not in self.account_id_to_acc:
+                self.logger.error(f"[update_order] can't_parse_trade_acc {trade_acc}")
+                return 
             acc = self.account_id_to_acc[trade_acc]
             order_info_collection = self.order_info_db[acc]['atx_order']
             query = {'oid' : oid}
@@ -716,7 +719,7 @@ if __name__ == "__main__":
     parser = argparse.ArgumentParser(description=description)
     
     parser.add_argument('-e' , '--end_time', dest='end_time', default='15:30')
-    _file_path = "C:/Users/Administrator/Desktop/ft_batandjson"
+    _file_path = "C:/Users/Administrator/Desktop/atx_batandconfig"
     parser.add_argument('-p', '--config_filepath', dest= 'config_filepath', default= _file_path)
     
     args = parser.parse_args()
