@@ -12,7 +12,7 @@ import getopt
 import argparse
 
 from pymongo import MongoClient, ASCENDING, DESCENDING
-from utils import decode_ft_flag,  decode_exchange_id
+from utils import decode_ft_flag, decode_exchange_id, send_to_feishu
 from logger import Logger
 # websocket-client
 
@@ -63,7 +63,7 @@ class FtServer(object):
             #self.db_client = MongoClient()
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=err)
             self.logger.error(f'[init] DB_connect_failed! (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -79,7 +79,9 @@ class FtServer(object):
         self.buy_orderlock = threading.Lock()
         self.sell_orderlock = threading.Lock()
         self.cancel_orderlock = threading.Lock()
-        self.order_dbids = []
+        self.buy_order_dbids = []
+        self.sell_order_dbids = []
+        self.cancel_order_dbids = []
         self.sids = []
         self.cancel_order_ids = []
         self.broker_id = {}
@@ -192,48 +194,24 @@ class FtServer(object):
             self.logger.info(
                 f"[get_account_info] (tg_name){self.tgnames} (logacc_name){self.log_account_names} (target_accountnames){self.target_account_names}")
             
-
-    def send_to_feishu(self, error):
-        try:
-            if self.feishu_url == None:
-                self.logger.info("[send_to_feishu] send_message_failed")
-                return
-            url = self.feishu_url
-
-            payload_message = {
-                "msg_type": "text",
-                "content": {
-                    "text": f'[ft.py] ERROR! (exception){error}'
-                }
-            }
-            headers = {
-                'Content-Type': 'application/json'
-            }
-
-            response = requests.request("POST", url, headers=headers, data=json.dumps(payload_message))
-            self.logger.info(f"[send_to_feishu] (response){response}")
-        except Exception as e:
-            err = traceback.format_exc()
-            exc_type, exc_value, exc_traceback = sys.exc_info()
-            traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
-            self.logger.error(f'[send_to_feishu] send_message_failed(exception){err}')
-
-    
     def date_change(self):
         try:
             while not self.is_stopped:
-                dt = datetime.now()
-                time_now = time.strftime("%H:%M", time.localtime())
-                if time_now == self.endtime:
+                time_now = datetime.now()
+                _dt_endtime = datetime.strptime(self.endtime, "%H:%M")
+                dt_endtime = datetime.combine(time_now, _dt_endtime.time())
+                if time_now > dt_endtime:
                     self.req_position()
-                    self.ft_close()                 
+                    msg = f"[date_change] close (now){time_now}"
+                    send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=msg)
+                    self.ft_close()         
                 else:
                     self.req_position()
-                    self.logger.info(f"[date_change] not_closed (now){dt}")
+                    self.logger.info(f"[date_change] not_closed (now){time_now}")
                 time.sleep(60)
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=err)
             self.logger.error(f'[date_change] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -243,7 +221,10 @@ class FtServer(object):
         self.ws.close()
         print (f"[ft_close] (close_time){self.endtime}")
         self.logger.info(f"[ft_close] (close_time){self.endtime}")
-        time.sleep(3)
+        
+        msg= f"[ft_close] (close_time){self.endtime}"
+        send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=msg)
+        
         os._exit(0)
 
     def req_position(self):
@@ -287,7 +268,7 @@ class FtServer(object):
                     self.logger.error(f"[req_position] failed (error_msg){data}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=err)
             self.logger.error(f'[req_position] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -306,7 +287,7 @@ class FtServer(object):
                 self.on_mudan_message(data)
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=err)
             self.logger.error(f'[on_message] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -477,7 +458,7 @@ class FtServer(object):
                 self.logger.info(f"[zidan_message] update_ft_trade (res){res} (local_id){local_id}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url,msg=err)
             self.logger.error(f'[on_zidan_message] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -514,7 +495,7 @@ class FtServer(object):
                     "ticker": ticker,
                     "cost": cost,  # 成本价
                     "td_pos_long": td_trade_vol,  # 今仓
-                    "yd_pos_long": yd_pos,  # 昨仓（可卖），挂止盈单后为0
+                    "yd_pos_long": yd_pos,  # 昨仓（可卖），挂止盈单后为0, use yd_pos_long 作为可卖
                     "td_pos_short": 0,
                     "yd_pos_short": 0,
                     "actual_td_pos_long": td_trade_vol,  # 实际的今仓
@@ -542,12 +523,14 @@ class FtServer(object):
                     self.logger.error(f"[update_position]no trade_vol (sid){sid}")
                     return
                 yd_pos = position['actual_yd_pos_long']
+                yd_pos_long = position['yd_pos_long']
                 amt = trade_amt
                 if side == 1:
                     td_trade_vol = position['actual_td_pos_long'] + msg['trade_vol']
                     mkt_value = position['mkt_value'] + amt
                 elif side == 2 :
                     td_trade_vol = position['actual_td_pos_long'] - msg['trade_vol']
+                    yd_pos_long -= msg['trade_vol']
                     mkt_value = position['mkt_value'] - amt
                 
                 order = self.oid_to_req[oid]
@@ -566,7 +549,7 @@ class FtServer(object):
                     "ticker": ticker,
                     "cost": cost,  # 成本价
                     "td_pos_long": td_trade_vol,  # 今仓
-                    "yd_pos_long": yd_pos,  # 昨仓（可卖），挂止盈单后为0
+                    "yd_pos_long": yd_pos_long,  # 昨仓（可卖），挂止盈单后为0
                     "td_pos_short": 0,
                     "yd_pos_short": 0,
                     "actual_td_pos_long": td_trade_vol,  # 实际的今仓
@@ -587,7 +570,7 @@ class FtServer(object):
                     f"[update_position] (res){res} (msg){position_msg}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[update_position] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -643,7 +626,7 @@ class FtServer(object):
                 f"[on_rtn_filled_order] order_filled (db_res){res} (msg){db_msg}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[update_order_filled] DB_connect_failed! (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -657,7 +640,7 @@ class FtServer(object):
         if type(error) == ConnectionRefusedError or type(error) == ConnectionAbortedError or type(error) == ConnectionResetError or type(error) == websocket._exceptions.WebSocketConnectionClosedException:
             self.reconnect_count += 1
             self.logger.info(f"[on_error] 正在尝试第{self.reconnect_count}次重连")
-            self.send_to_feishu(error)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=error)
             if self.reconnect_count < 100:
                 self.ws.close()
                 t = threading.Thread(target=self.run_socket)
@@ -699,7 +682,6 @@ class FtServer(object):
         signal.signal(signal.SIGTERM, self.signal_handler)
         signal.signal(signal.SIGINT, self.signal_handler)
         self.login()
-        #test
         self.req_position()
         self.monitor()
 
@@ -730,10 +712,13 @@ class FtServer(object):
                 trade_acc = myacc['trade_acc']
                 self.broker_id[trade_acc] = myacc['broker_id']
                 self.logger.info(f"(trade_acc){trade_acc} broker_id{self.broker_id[trade_acc]}")
+            
+            _msg = f"[login] logged (token){self.token} (data){data}"
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=_msg)
                 
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[login] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -789,8 +774,8 @@ class FtServer(object):
                             self.logger.warning("[monitor_buy_order] no_buy_target")
                             continue
                         for target in buy_targets:
-                            if target['_id'] not in self.order_dbids:
-                                self.order_dbids.append(target['_id'])
+                            if target['_id'] not in self.buy_order_dbids:
+                                self.buy_order_dbids.append(target['_id'])
                                 self.req_buy_order_insert(target)
                 for acc in self.accounts_run:
                     target_accountname = self.target_account_names[acc]
@@ -802,13 +787,13 @@ class FtServer(object):
                                 "[monitor_sell_order] no_sell_target")
                             continue
                         for sell_target in sell_targets:
-                            if sell_target['_id'] not in self.order_dbids:
-                                self.order_dbids.append(sell_target['_id'])
+                            if sell_target['_id'] not in self.sell_order_dbids:
+                                self.sell_order_dbids.append(sell_target['_id'])
                                 self.req_sell_order_insert(sell_target)
                 time.sleep(self.scan_interval)
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[monitor_algo_order_insert] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -971,7 +956,7 @@ class FtServer(object):
             
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[req_buy_order_insert] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -984,7 +969,7 @@ class FtServer(object):
             self.logger.info(f"[on_req_buy_order_insert] ft_proposed_target_collection (insert_res){insert_res} (order_msg){order_msg}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[insert_ft_proposed_target] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -1038,7 +1023,7 @@ class FtServer(object):
                 f"[rtn_order_inserted] order_inserted (db_res){res} (db_msg){db_msg}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[rtn_order_insert] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -1056,8 +1041,8 @@ class FtServer(object):
                                 "[monitor_sell_order] no_sell_target")
                             continue
                         for sell_target in sell_targets:
-                            if sell_target['_id'] not in self.order_dbids:
-                                self.order_dbids.append(sell_target['_id'])
+                            if sell_target['_id'] not in self.cancel_order_dbids:
+                                self.cancel_order_dbids.append(sell_target['_id'])
                                 self.req_sell_order_insert(sell_target)
                         result = self.orderinfo_db["sell_target"].delete_many(query)
                         self.logger.info(
@@ -1065,7 +1050,7 @@ class FtServer(object):
                 time.sleep(self.scan_interval)
         except Exception as e:
                 err = traceback.format_exc()
-                self.send_to_feishu(err)
+                send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
                 self.logger.error(f'[rtn_order_insert] (exception){err}')
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -1213,7 +1198,7 @@ class FtServer(object):
             self.logger.info(f"[on_req_buy_order] delete (target){obj}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[req_sell_order_insert] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -1239,7 +1224,7 @@ class FtServer(object):
                 time.sleep(self.scan_interval)
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[monitor_cancel_order] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -1300,7 +1285,7 @@ class FtServer(object):
                 f"[rtn_order_canceled] order_canceled (db_res){res} (db_msg){db_msg}")
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[rtn_order_cancel] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
@@ -1340,7 +1325,7 @@ class FtServer(object):
             # cancel success 所以回rtn_order,实际什么时候撤单不确定
         except Exception as e:
             err = traceback.format_exc()
-            self.send_to_feishu(err)
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=err)
             self.logger.error(f'[req_cancel_order] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
