@@ -9,9 +9,10 @@ import traceback
 import getopt
 import argparse
 from pathlib import Path
+import requests
 
 from pymongo import MongoClient, ASCENDING, DESCENDING
-from utils import decode_ft_flag,  decode_exchange_id
+from utils import decode_ft_flag,  decode_exchange_id, send_to_feishu
 from logger import Logger
 
 
@@ -56,15 +57,15 @@ class FtTool(object):
 
         
         #test for req_position
-        self.db_client_test = MongoClient("127.0.0.1", 27017, connectTimeoutMS=10000)
-        self.test_trading_account = self.db_client_test['tradingAccount']
+        #self.db_client_test = MongoClient("127.0.0.1", 27017, connectTimeoutMS=10000)
+        #self.test_trading_account = self.db_client_test['tradingAccount']
 
         self.get_acccount_info()
     
     def load_ft_setting(self, config_filename):
         try:
             #f = open(config_filename, encoding="utf-8")
-            f = open(config_filename)
+            f = open(config_filename, encoding='gbk')
             setting = json.load(f)
             
             
@@ -155,21 +156,31 @@ class FtTool(object):
             self.logger.error(f'[get_account_info] (exception){err}')
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
 
+
     def start(self):
         
         #trade_export_path = r'C:\Users\Administrator\Downloads\ft_client_win_1.6.4\ft_client_win\export\20221201\20221201_326000024361_trade.csv'
         if self.sync_trade == True:
             self.logger.info(f"in get_trade")
             self.get_trade_from_csv()
+            _msg = "[get_trade_from_csv] executed"
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=_msg)
+            
             
         if self.sync_trade_in_trading == True:
             self.get_trade_from_csv_in_trading()
+            _msg = "[get_trade_from_csv_in_trading] executed"
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=_msg)
         
         if self.sync_pos == True:
             self.date_change()
+            _msg = "[date_change] executed"
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=_msg)
         
         if self.sync_add == True:
-            self.sync_position_add()            
+            self.sync_position_add()
+            _msg = "[sync_add] executed"   
+            send_to_feishu(logger=self.logger, feishu_url=self.feishu_url, msg=_msg)
         
     def get_trade_from_csv(self):
         try:
@@ -384,7 +395,7 @@ class FtTool(object):
             self.logger.error(f'[get_trade_from_csv_in_trading] (exception){err}')
             exc_type, exc_value, exc_traceback = sys.exc_info()
             traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
-                  
+
     def date_change(self):
         try:
                 dt = datetime.now()
@@ -445,10 +456,10 @@ class FtTool(object):
                                     exchange = one_trade['exchange']
                                     position_for_update = equity_position_collection.find_one(query_by_sid)
                                     if position_for_update is not None:
-                                        yd_pos_long = position_for_update['yd_pos_long'] + position_yd_vol
+                                        #yd_pos_long = position_for_update['yd_pos_long'] + position_yd_vol
                                         actual_yd_pos_long = position_for_update['actual_yd_pos_long'] + position_yd_vol
                                         data_to_update ={
-                                            'yd_pos_long' : yd_pos_long,
+                                            'yd_pos_long' : actual_yd_pos_long,
                                             'actual_yd_pos_long' : actual_yd_pos_long
                                         }
                                         new_data = {"$set": data_to_update}
@@ -461,7 +472,7 @@ class FtTool(object):
                                 self.logger.info(f"delete_position (sid){sid}")
                             else: #找不到sync_trade 直接将add持仓分到所有持仓上
                                 self.logger.info("[sync_position_add] nothing")
-                                yd_pos_long = position['yd_pos_long']
+                                #yd_pos_long = position['yd_pos_long']
                                 actual_yd_pos_long = position['actual_yd_pos_long']
                                 if actual_yd_pos_long > 0:
                                     query_positions = {'accountName': account_name, 'ticker': ticker, 'sid' :  {'$ne' : sid}}
@@ -469,12 +480,12 @@ class FtTool(object):
                                     if position_for_update == None:
                                         self.logger.warning("[sync_position_add] one_pos")
                                     else:
-                                        yd_pos_long = position_for_update['yd_pos_long'] + actual_yd_pos_long
+                                        #yd_pos_long = position_for_update['yd_pos_long'] + actual_yd_pos_long
                                         actual_yd_pos_long = position_for_update['actual_yd_pos_long'] + actual_yd_pos_long
                                         _id = position_for_update['_id']
                                         _query = {'_id': _id}
                                         data_to_update ={
-                                            'yd_pos_long' : yd_pos_long,
+                                            'yd_pos_long' : actual_yd_pos_long,
                                             'actual_yd_pos_long' : actual_yd_pos_long
                                         }
                                         new_data = {"$set": data_to_update}
@@ -504,7 +515,7 @@ class FtTool(object):
                                                 query_for_delete = {'_id': _id}
                                                 res = equity_position_collection.delete_one(query_for_delete)
                                                 self.logger.info(f'[sync_position_add] delete (res){res} ')
-                                                yd_pos_long = pos
+                                                #yd_pos_long = pos
                                                 
                                 query_for_delete_add = {'sid' : sid , 'accountName' : account_name}
                                 equity_position_collection.delete_one(query_for_delete_add)
